// This  skeleton file was automatically created by 
//            pkgmaker 0.1
//            Thursday, December 11 1997
// 
// Module: ImageToColumn
// Author: Hassan Siddiqui
//
// Description: Expands a given row range (first dimension) 
// of the input image and writes 
// the row(s) out to separate columns in the output table. 
// 
// Comments: 
//  
// Modifications: See ChangeLog
// 
// $Id: ImageToColumn.h,v 1.5 2002/02/12 08:32:19 gvacanti Exp $

// General classes
#include <iostream>

// SAS-specific classes
#include "Dal.h"
#include "FileLocator.h"
#include "Error.h"

// General utility functions
#include "Tracer.h"
#include "stringutil.h"


// ---------------------------------------------------

// Class ImageToColumn

// ---------------------------------------------------

class ImageToColumn
{
public:

  static void addPositionColumn(Table* table)
    {
      PROC;

      // Add a column to keep track of the row numbers. 
      // This is a temporary addition to enable the SAS task dsplot
      // to be used without too much fiddling about.

      Column* numberCol = table->addColumn("POSITION", 
						 Column::Int16, 
						 "position in array ", 
						 "");

      int16* numberData = numberCol->data()->int16Data();

      
      for (unsigned int element = 0; 
	   element < table->rows();
	   ++element)
	{
	  numberData[element] = element; 
	}

      ENDPROC;
    }
		      

  static void convert(DataSet* outSet,
		      const Array* imageArray,
		      const string& name,
		      const vector<unsigned int>& rows,
		      const string rowPrefix="ROW_")
    {
      PROC;
      
      Table* outputTable = outSet->addTable(name, 
					    (imageArray->size())[1] );

      addPositionColumn(outputTable);

      convert(imageArray, outputTable, rows, rowPrefix);

      ENDPROC;
    }

		      
  static void convert(const Array* imageArray, 
		      Table* outputTable,
		      const vector<unsigned int>& rows,
		      const string rowPrefix)
    {
      PROC;

      // This function can only handle 2-d arrays

      assert(imageArray->size().size() == 2);


      // This function can only handle Real32 data types

      assert(imageArray->dataType() == Array::Real32);

      // Check sizes of outputTable and imageArray
      assert ( outputTable->rows() == (imageArray->size())[1] );


      // Get pointer to image array which provide the necessary 
      // information with which to expand the row.

      real32* arrayData = imageArray -> data() -> real32Data();
      // REMOVE THIS
      assert(arrayData);


      // Loop over all the rows
      // real32* outputData;

      vector<long unsigned int> pos;

      for (vector<unsigned int>::const_iterator rowIndex = rows.begin(); 
	   rowIndex != rows.end(); 
	   ++rowIndex)
	{
	  // Add a column, representing the row information, to the output table

	  Column* outputCol = outputTable->addColumn(rowPrefix + toString(*rowIndex), 
						     Column::Real32, 
						     "Expanded data for row", 
						     "");

	  real32* outputData = outputCol->data()->real32Data();
	  
	  // Loop over elements (second dimension)

	  for (unsigned int element = 0; 
	       element < (imageArray->size())[1];
	       ++element)
	    {
	      
	      // Add matrix element to output element.
	      // NOTE: assume outputData intialised to 0

	      pos.erase(pos.begin(), pos.end());
	      
	      // Check that row number is within range 1 to no. rows inclusive
	      // NB: a zero index is not valid here, 
	      // in line with FTOOLS approach.
	      
	      if (*rowIndex < 1 || *rowIndex > (imageArray->size())[1] )
		{
		  errstr << "The requested row number: " << *rowIndex 
			 << " is out of range ( 1 - " 
			 << (imageArray->size())[1] << " ) "
			 << Sas::error("rowIndexOutOfRange"); 

		}

	      pos.push_back(*rowIndex - 1);
	      pos.push_back(element);

// 	      cerr << " row " << *rowIndex
// 		   << " element " << element
// 		   << " array value " << 
// 		arrayData[imageArray->data()->position(pos)]
// 		   << endl;

	      assert(element < outputCol->rows());

	      outputData[outputCol->data()->position(element)]
		= arrayData[imageArray->data()->position(pos)]; 

	    }
	  
	  //	  delete outputData;
	  
	}

      ENDPROC;
    }


private:

  // Suppress default constructor
  ImageToColumn();

}; 
