// This  skeleton file was automatically created by 
//            pkgmaker 0.1
//            Thursday, December 11 1997
// 
// Module: RmfRowToColumn
// Author: Hassan Siddiqui
//
// Description: Expands a given row range of the input rmf and writes 
// the sum out to a column in the output table. 
// 
// Comments: 
//  
// Modifications: See ChangeLog
// 
// $Id: RmfRowToColumn.h,v 1.8 2002/02/12 08:32:19 gvacanti Exp $

#ifndef RmfRowToColumn_H
#define RmfRowToColumn_H

// General classes
#include <iostream>

// SAS-specific classes
#include "param.h"
#include "Dal.h"
#include "FileLocator.h"

// General utility functions
#include "Tracer.h"
#include "stringutil.h"

// Local classes
#include "VectorColumn.h"
#include "LabelledInt.h"
#include "RowRetriever.h"


// --------------------------------------------------

// Class RowRange

// This is a temporary structure, which MAY be replaced later
// by the RowRange structure in the selectlib package once the latter
// has been disassociated from selectlib.

// --------------------------------------------------

// class RowRange
// {
// public:
  
//   RowRange(unsigned long startRow, unsigned long stopRow)
//     {
//       PROC;

//       assert (stopRow >= startRow);
//       _startRow = startRow;
//       _stopRow  = stopRow;

//       ENDPROC;
//     }

//   const unsigned long startRow() const {PROC; RETURN  _startRow;}
//   const unsigned long stopRow() const  {PROC; RETURN  _stopRow;}

// private:
  
//   RowRange();

//   unsigned long	_startRow, _stopRow;	// from row #startRow until
//                                         // row #stopRow (inclusive)
// };


/* ---------------------------------------------------
 *
 * class RmfRowToColumn
 *
 * --------------------------------------------------- */

class RmfRowToColumn
{
public:

  // Dummy fn. For testing readrmf.
  //   static void convert(const Table*, 
  // 		      Table*, 
  // 		      const RowRange&, 
  // 		      const string&)
  //     {}



  static void convert(const Table * rmfTable, 
		      DataSet * outputSet,
		      const string & tableName,
		      const vector<LabelledInt>& rows)
    {
      PROC;

      /* Create Rowretriever object */
      RowRetriever rowretriever(rmfTable);

      /* Add a table to the output dataset */
      Table* outputTable = 
	outputSet->addTable(tableName, rowretriever.ndetchans());

      /* Add a column to keep track of the channel numbers. 
       * This enables the SAS task dsplot
       * to be used without too much fiddling about. */

      Column* numberCol = outputTable->addColumn("CHANNEL", 
						 Column::Int16, 
						 "CHANNEL NUMBER", 
						 "");

      int16* numberData = numberCol->data()->int16Data();

      for (unsigned int element = 0; 
	   element < outputTable->rows(); 
	   ++element)
	{
	  /* keep track of row no.s; see above. */
	  numberData[element] = element; 
	}


      /* Loop over all the rows */
      
      for (vector<LabelledInt>::const_iterator rowItr = rows.begin(); 
	   rowItr != rows.end(); 
	   ++rowItr)
	{
	  /* Add a column, representing the rmf information, to the output table */

	  Column* outputCol = outputTable->addColumn((*rowItr).name(), 
						     Column::Real32, 
						     (*rowItr).label(),
						     ""
						     );

	  real32* outputData = outputCol->data()->real32Data();
	  
	  vector<double> data = rowretriever.rowData( *rowItr );
	  
	  assert(data.size() == outputTable->rows());

	  for (unsigned int element = 0; 
	       element < outputTable->rows(); 
	       ++element)
	    {
	      outputData[element] = data[element];
	    }

	}

      ENDPROC;
    }


private:

  /* Suppress default constructor */
  RmfRowToColumn();

}; 

#endif
