// This  skeleton file was automatically created by 
//            pkgmaker 0.1
//            Thursday, December 11 1997
// 
// Module: RowRetriever
// Author: Hassan Siddiqui
//
// Description: Expands a given row range of the input rmf. 
// 
// Comments: 
//  
// Modifications: See ChangeLog
// 
// $Id: RowRetriever.h,v 1.4 2002/02/12 08:32:19 gvacanti Exp $

#ifndef RowRetriever_H
#define RowRetriever_H

// General classes
#include <iostream>

// SAS-specific classes
#include "Dal.h"

// General utility functions
#include "Tracer.h"
#include "stringutil.h"

// Local classes
#include "VectorColumn.h"
#include "LabelledInt.h"

class RowRetriever
{

public:

  /* Constructor */
  RowRetriever(const Table * rmfTable)
    {
      PROC;
      
      /* Get pointers to those columns in the 
       * rmf table which provide the necessary 
       * information with which to expand the row.*/

      _matrixContainer = new VectorColumnContainer (rmfTable->column("MATRIX")); 
      _n_grpContainer  = new VectorColumnContainer (rmfTable->column("N_GRP"));
      _f_chanContainer = new VectorColumnContainer (rmfTable->column("F_CHAN"));
      _n_chanContainer = new VectorColumnContainer (rmfTable->column("N_CHAN"));

      _elowContainer   = new VectorColumnContainer (rmfTable->column("ENERG_LO"));
      _ehighContainer  = new VectorColumnContainer (rmfTable->column("ENERG_HI"));

      /* Misc attribute info. */
      _ndetchans = rmfTable->attribute("DETCHANS")->asInt();
      
      _lo_thres=0;
      if (rmfTable->hasAttribute("LO_THRES"))
	{
	  _lo_thres=rmfTable->attribute("LO_THRES")->asReal();
	}


//       /* Find out the maximum number of elements over 
//        * the row range in question. 
//        * This should be less than numberDetChans. */
      
//       unsigned int maxElements = 0;
      
//       /* Loop over all the rows */

//       for (vector<LabelledInt>::const_iterator rowIndex = rows.begin(); 
// 	   rowIndex != rows.end(); 
// 	   ++rowIndex)
// 	{

// 	  if ((*rowIndex).value() >= rmfTable->rows())
// 	    {
// 	      errstr << " Row index ( " << (*rowIndex).value() 
// 		     << " ) is out of bounds of table " 
// 		     << Sas::error("RowIndexOutOfBounds");

// 		ALTRETURN;
// 	    }

// 	  unsigned int lastgroup = (unsigned int) _n_grpContainer->value((*rowIndex).value(), 0) - 1;

// // 	  cout << "row index " << (*rowIndex).value() << endl;
// // 	  cout << "last group " << lastgroup << endl;

// // 	  cout << " fchan " << _f_chanContainer->value((*rowIndex).value(), lastgroup)
// // 	       << " nchan " << _n_chanContainer->value((*rowIndex).value(), lastgroup)
// // 	       << endl;

// 	  unsigned int numberOfElements = 
// 	    (unsigned int) _f_chanContainer->value((*rowIndex).value(), lastgroup) +
// 	    (unsigned int) _n_chanContainer->value((*rowIndex).value(), lastgroup);

// 	  if (numberOfElements > maxElements)
// 	    {
// 	      maxElements=numberOfElements;
// 	    }
// 	}
      
// //       cout << "max no of elements in row range is " << maxElements << endl;


//       /* Now check if everything is consistent */
//       if (maxElements > (unsigned int) numberDetChans)
// 	{
// 	  errstr << " Number of elements in matrix (" << maxElements 
// 		 << ") is greater than the number of detector channels ("
// 		 << numberDetChans
// 		 << Sas::error("elementsExceedsChannelRange");
	  
// 	  ALTRETURN;
// 	}
      

      ENDPROC;
    };

  /* Retrieve an expanded row from the table */

  vector<double> rowData( const LabelledInt & rowIndex ) const
    {
      /* Write some diagnostic stuff */

      cout << " Row "   << (rowIndex).value() 
	   << " \""     << (rowIndex).label() << "\" " 
	   << " elow "  << _elowContainer->value((rowIndex).value(), 0)
	   << " ehigh " << _ehighContainer->value((rowIndex).value(), 0)
	   << endl;

      RETURN rowData(rowIndex.value());
    }

  vector<double> rowData( const unsigned int & row) const
    {
      vector<double> outVector;

      /* A variable to keep track of the channel position */
      unsigned int channel = 0;

      /* Define a variable to follow position along matrix column */
      unsigned int matrixPosition = 0;

      /* Define a variable to mark the start of a gap between groups.
	 This is set initially to the start of the channel range. */
      unsigned int startOfGap = 0;

	  
      /** Loop over all the groups */

      /* Check if +ve */
      assert ( _n_grpContainer->value(row, 0) >= 0 ); 
      for (unsigned int group = 0; 
	   (int) group < _n_grpContainer->value(row, 0); 
	   ++group)
	{
	  /* Loop over the element (pha channel) range */
	  
	  /* again, check if +ve */
	  assert (_f_chanContainer->value(row, group) >=0); 

	  unsigned int fchan = 
	    (unsigned int) _f_chanContainer->value(row, group);
	  
	  /* Fill cells in vector from startOfGap up to but excluding
	     fchan with low_threshold */
	  for (unsigned int i = startOfGap;
	       i < fchan - 1;
	       ++i)
	    {
	      outVector.push_back(_lo_thres);
	      channel++;
	    }


// 	  cout 
// 	    << "row " << row 
// 	    << " fchan " << fchan
// 	    << " nchan " << _n_chanContainer->value(row, group)
// 	    << " number elements " 
// 	    << _f_chanContainer->value(row, group) + 
// 	    _n_chanContainer->value(row, group) - 1 << endl;

	  for (unsigned int element = 0;
	       (int) element < _n_chanContainer->value(row, group); 
	       ++element)
	    {
	      /* Add matrix element to output element. */
	      
	      double matrixValue =  
		_matrixContainer->value(row, matrixPosition);

	      outVector.push_back(matrixValue);
		  
// 	      cout
// 		<< " group " << group
// 		<< " group element " << element
// 		<< " matrix position " << matrixPosition
// 		<< " matrix value " << matrixValue
// 		<< " matrix size " << outVector.size()
// 		<< endl;
		  
	      /* Increment matrix position */
	      matrixPosition++;

	    }
	}

      int outsize = outVector.size();

      /* Check if number of cells read out does not exceed number of channels */
      if ( outsize > _ndetchans)
	{
	  errstr << " For row " << row 
		 << " there are "<< outsize 
		 << " elements, exceeding NDETCHANS ("
		 << _ndetchans << ")"
		 << Sas::error("RowIndexOutOfBounds");
	  
	  vector<double> nullVector;
	  
	  ALTRETURNV(nullVector);
	}

      /* Pad out end of vector with _lo_thres values */
      for (int i = outsize; i < _ndetchans; ++i)
	{
	  outVector.push_back(_lo_thres);
	}

//       cout << " outsize " << outVector.size() 
// 	   << " ndetchans " << _ndetchans << endl;

      RETURN outVector;
    }

  int ndetchans() const
    {
      PROC;
      RETURN _ndetchans;
    }


  ~RowRetriever()
    {
      delete _matrixContainer; 
      delete _n_grpContainer;
      delete _f_chanContainer;
      delete _n_chanContainer;
      delete _elowContainer;
      delete _ehighContainer;
    }

private:

  VectorColumnContainer * _matrixContainer; 
  VectorColumnContainer * _n_grpContainer;
  VectorColumnContainer * _f_chanContainer;
  VectorColumnContainer * _n_chanContainer;

  /* For diagnostic purposes */
  VectorColumnContainer * _elowContainer;
  VectorColumnContainer * _ehighContainer;
  
  double _lo_thres;
  int    _ndetchans;
  
};

#endif
