// This  skeleton file was automatically created by 
//            pkgmaker 0.1
//            Thursday, December 11 1997
// 
// Module: VectorColumn
// Author: Hassan Siddiqui
//
// Modifications: See ChangeLog
// 
// $Id: VectorColumn.h,v 1.5 2002/02/12 08:32:19 gvacanti Exp $

#ifndef VectorColumn_H
#define VectorColumn_H

// General classes
#include <iostream>

// SAS-specific classes
#include "Dal.h"

// General utility functions
#include "Tracer.h"


// ---------------------------------------------------

// class VectorColumn

// Abstract class for VectorColumn objects

// ---------------------------------------------------

class VectorColumn
{
public:

  /* Return a value at the position of interest.
   * Current implementation handles only data of type real32. */
  virtual double value ( unsigned long row, unsigned long element ) const = 0;

//   /* For scalar columns */
//   virtual double value ( unsigned long /* row */) const
//     {
//       PROC;
//       errstr << " function value() : cannot be used for this column type " 
// 	     << error("ColumnDataNotScalar");
      
//       RETURN 0.;
//     }


  virtual ~VectorColumn(){PROC; ENDPROC;}
};


// ---------------------------------------------------

// class FixedVectorColumn

// Handles Fixed-length Vector columns

// ---------------------------------------------------
class FixedVectorColumn :  public VectorColumn
{

public:

  FixedVectorColumn ( const Column * column )
    {
      PROC;

      _columnData = column -> data();

      switch(_columnData->dataType())
	{
	case (Column::Real32):
	  _real32data = _columnData -> real32Data();
	  break;

	case (Column::Int16):
	  _int16data = _columnData -> int16Data();
	  break;

	default:
	  errstr << " FixedVectorColumn : datatype of " 
		 << (unsigned long)_columnData->dataType()
		 << " Is not supported. "
		 << Sas::error("ColumnDataTypeNotSupported");
	}

      ENDPROC;
    }
  
  
  ~FixedVectorColumn()
    {
      PROC;
      delete _columnData; _columnData = 0;
      ENDPROC;
    }

  double value ( unsigned long row, unsigned long element) const
    {
      PROC;

      assert(_columnData->elements() > element);
      assert(_columnData->parent()->rows() > row);


      switch(_columnData->dataType())
	{
	case (Column::Real32):
	  ALTRETURNV( _real32data[_columnData->position( row, element )] );

	case (Column::Int16):
	  ALTRETURNV( _int16data[_columnData->position( row, element )] );

	default:
	  /* Shouldn't get to this 
	     unless the two switch statements in this class 
	     are out of sync. */
	  errstr << " FixedVectorColumn : datatype of " 
		 << (unsigned long)_columnData->dataType()
		 << " Is not supported. THIS IS A PROGRAMMING ERROR. "
		 << Sas::error("ColumnDataTypeNotSupported");
	}

      RETURN 0;
    }

//   double value ( unsigned long row ) const
//     {
//       PROC;
//       if (_columnData->elements() != 1)
// 	{
// 	  errstr << " FixedVectorColumn::value() : column is not scalar (it has " 
// 		 << (unsigned long)_columnData->elements()
// 		 << " elements) and so cannot be used in this function. "
// 		 << error("ColumnDataNotScalar");
// 	  ALTRETURNV(0.);
// 	}
      
//       RETURN value(row, 0);
//     }

private:

  FixedVectorColumn();

  const ColumnData * _columnData;
  const real32 * _real32data;
  const int16 * _int16data;

};


// ---------------------------------------------------

// class VariableVectorColumn

// Handles Variable-length Vector columns

// ---------------------------------------------------
class VariableVectorColumn :  public VectorColumn
{

public:

  VariableVectorColumn ( const Column * column ): _column(column)
    {
      PROC;
      ENDPROC;
    }
  
  
  ~VariableVectorColumn()
    {
      PROC;
      ENDPROC;
    }

  double value ( unsigned long row, unsigned long element) const
    {
      PROC;

      assert(_column->cellType() !=  Column::Fixed);

      CellData * cellData = _column -> cellData( row );

//       cout << "cellData->elements() " << cellData->elements() << endl;
      assert ( cellData->elements() > element );


      switch ( cellData -> dataType())
	{
	case (Column::Real32):
	  ALTRETURNV((cellData -> real32Data())[element]);

	case (Column::Int32):
	  ALTRETURNV((cellData -> int32Data())[element]);

	case (Column::Int16):
	  ALTRETURNV((cellData -> int16Data())[element]);

	default:
	  errstr << " VariableVectorColumn : datatype of " 
		 << (unsigned long)cellData ->dataType()
		 << " Is not supported. "
		 << " For column " << _column->name()
		 << Sas::error("ColumnDataTypeNotSupported");
	}

      RETURN 0;
    }

private:

  VariableVectorColumn();

  const Column * _column;

};

// ---------------------------------------------------

// class VectorColumnContainer

//  Contains either a
//  fixed-length or variable length column data. 
//  The type is invisible to user.

// ---------------------------------------------------

class VectorColumnContainer : public VectorColumn
{
public:

  VectorColumnContainer( const Column * column )
    {
      PROC;

      /* Check if fixed-length or variable */
      if ( column->cellType() == Column::Fixed )
	{
	  _vcol  = new FixedVectorColumn(column);
	  ALTRETURN;
	}
      
      _vcol = new VariableVectorColumn(column);
    }

  ~VectorColumnContainer()
    {
      PROC;
      delete _vcol; _vcol = 0;
      ENDPROC;
    }

  double value ( unsigned long row, unsigned long element) const
    {
      PROC;
      RETURN _vcol->value(row, element);
    }

private:

  /* Disallow default constructor */
  VectorColumnContainer();

  VectorColumn * _vcol;
};



#endif
