// This  skeleton file was automatically created by 
//            pkgmaker 0.1
//            Thursday, December 11 1997
// 
// Module: colmerge
//
// Description: merges columns together into a single dataset.
// if the column lengths are different, the shorter columns are padded out with
// zeroes.
//
// Only works with scalar numeric columns.
//
// Author: Hassan Siddiqui
//
// Modifications: See ChangeLog
// 
// $Id: colmerge.cc,v 1.4 2002/02/12 08:32:19 gvacanti Exp $

/* General classes */
#include <iostream>

/* SAS-specific classes */
#include "param.h"
#include "Dal.h"
#include "GenericOperation.h"

/* General utility functions */
#include "Tracer.h"
#include "stringutil.h"

/* Utility classes */

/* Local */
#include "colmerge.h"


/* Macro for copying columns of differing types.
   modified from CopyUtils.h.
   A scalar column is copied to a new column, the remaining rows padded with
   zeroes.
*/
#define COPYCOLUMNDATA(TYPE)\
TYPE* fromData = fromColumn ->data() -> TYPE ## Data();\
TYPE* toData   = toColumn   ->data() -> TYPE ## Data();\
for (unsigned long index = 0; index < fromColumn->rows(); ++index)\
{ toData[index] = fromData[index];}\
for (unsigned long index = fromColumn->rows(); index < toColumn->rows(); ++index)\
{ toData[index] = 0;}

/* ---------------------------------------------------
 *
 * Main function
 *
 * --------------------------------------------------- */

void colmerge()
{
  PROC;

  /* Get list of columns, specified according to set:table:column format.  */
  vector<string> tablelist = stringListParameter("tables");

  if (tablelist.size() < 2)
    {
      errstr << "there are less than two tables specified by tables parameter " 
	<< Sas::error("notEnoughTables");    

      ALTRETURN;
    }

  /* Find number of rows in largest table */
  unsigned int max_rows = 0;
  for (vector<string>::const_iterator i = tablelist.begin();
       i != tablelist.end();
       ++i)
    {
      /* Access table specified by dataset */
      DataSet * set = dataSetServer->open(*i, DataSet::Read);
      Table * table = set -> table(*i);
      unsigned int nrows = table->rows();

      dataSetServer->close(set);

      if (nrows > max_rows)
	{
	  max_rows = nrows;
	}
    }

  // Open the output datset
  string outsetStr = stringParameter("outset");
 
  DataSet* outSet = 
    dataSetServer->open(outsetStr, DataSet::Create);

  // Add a table to the output dataset
  Table* outputTable = outSet->addTable("MERGED_TABLE", max_rows);


  unsigned int table_number  = 0;
  for (vector<string>::const_iterator i = tablelist.begin();
       i != tablelist.end();
       ++i)
    {
      table_number++;

      DataSet * set    = dataSetServer->open(*i, DataSet::Read);
      Table   * table  = set -> table(*i);


      /* Access columns */
      
      for (unsigned int colindex = 0; 
	   colindex < table->columns(); 
	   ++colindex) 
	{

	  Column  * fromColumn = table->column(colindex);
	  ColumnData * fromData = fromColumn->data();

	  /* Check that column is scalar */
	  if ((fromData->size()).size() != 1 && (fromData->size())[0] != 1)
	    {
	      errstr << " Vector columns not supported." 
		     << Sas::error("VectorColumnsNotSupported");
	    }
      	  /* Add a new column to o/p dataset */
      
	  string colname =  "T" + toString(table_number) + fromColumn->name();
      
	  Column * toColumn = outputTable->addColumn(colname, 
						     fromColumn->dataType(), 
						     fromColumn->label(), 
						     fromColumn->units());

	  /* Copy fromColumn to toColumn. 
	     See COPYCOLUMNDATA macro for more info. */
	  GENERIC_OPERATION(COPYCOLUMNDATA, fromColumn->dataType()); 
 	}

      /* Close input set. */
      dataSetServer->close(set);

    }

  // Close output dataset
  dataSetServer->close(outSet);


  ENDPROC;
} 
