// This  skeleton file was automatically created by 
//            pkgmaker 0.1
//            Thursday, December 11 1997
// 
// Module: comparermf
// Author: Hassan Siddiqui
//
// Description: Compares rmf with that obtained directly from CAL.
// 
// Comments: 
//  
// Modifications: See ChangeLog
// 
// $Id: comparermf.cc,v 1.8 2002/02/12 08:32:19 gvacanti Exp $

// General classes
#include <iostream>

// SAS-specific classes
#include "param.h"
#include "Dal.h"
#include "CalServer.h"
#include "Redistribution.h"

// General utility functions
#include "Tracer.h"

// Utility classes
#include "RowRetriever.h"

// Local
#include "comparermf.h"

// ---------------------------------------------------

// Main function

// ---------------------------------------------------

/* NOTE: row indices correspond to FTOOLS approach, in that the first
 * row has an index of 1.
 */

void comparermf()
{
  PROC;

  /* Get tolerance level */
  double tolerance = realParameter("tolerance");

  if (tolerance < 0. || tolerance > 1.0)
    {
      errstr << " Tolerance value of " << tolerance 
	     << " is outside range of 0 <= tolerance <= 1."
	     << Sas::error("ToleranceOutsideRange");
      ALTRETURN;
    }

  /* Set up CAL */
 
  calServer.setInstrument(Xmm::EMOS1);
  calServer.state()->ccd()->set(EMOScam::CCD_1);
  calServer.state()->node()->set(EMOScam::PRIMARY_NODE);

  CanonicalRedist * rmf = calServer.getAtom(rmf);

  errstr << " Number of energy bins " << rmf->eBounds().bins() 
	 << Sas::message(Msg::App,Msg::Verbose);
  
  /* Access rmf table */

  DataSet* rmfSet = 
    dataSetServer->open(stringParameter("set"), DataSet::Read);

  Table* rmfTable = rmfSet -> table(stringParameter("set"));

  /* Create Rowretriever object */
  RowRetriever * rowretriever = new RowRetriever(rmfTable);


  for ( unsigned int row = 0; 
	row < rmfTable->rows(); 
	++row )
    {
      
      if (!(row%100))
	{
 	  errstr << " Getting row " << row << " .." 
		 << Sas::message(Msg::App,Msg::Verbose);
	}

      vector<double> rmfdata = rowretriever->rowData(row);
      vector<double> caldata = rmf->spectrum(row);
      
      /* Check the rows */
      if (rmfdata.size() != caldata.size())
	{
	  errstr << " RMF row size (" << rmfdata.size() 
		 << ") differs from CAL row size (" << caldata.size()
		 << ")"
		 << Sas::error("RowSizesDiffer");
	  ALTRETURN;
	}

      for (unsigned int element = 0; 
	   element < rmfdata.size();
	   ++element)
	{
	  /* Get tolerance range */
	  double calelement = caldata[element];
	  double calLow     = calelement * (1 - tolerance);
	  double calHigh    = calelement * (1 + tolerance);
	  
	  double rmfelement = rmfdata[element];
	  

	  if (rmfelement < calLow || rmfelement > calHigh)
	    {
	      errstr << " RMF element " << element
		     << " of value " << rmfelement
		     << " differs from tolerance range defined by corresponding CAL element (" << calelement
		     << ")"
		     << Sas::error("ElementsDiffer");
	      ALTRETURN;
	    }

	}
      
    }


  /* Cleanup */
  delete rowretriever;
  dataSetServer->close(rmfSet);


  ENDPROC;
} 
