// This  skeleton file was automatically created by 
//            pkgmaker 0.1
//            Thursday, December 11 1997
// 
// Module: getcalresponse
// Author: Hassan Siddiqui
//
// Modifications: See ChangeLog
// 
// $Id: getcalresponse.cc,v 1.7 2002/02/12 08:32:20 gvacanti Exp $


// General classes
#include <iostream>

// SAS-specific classes
#include "param.h"
#include "Dal.h"
#include "CalServer.h"
#include "Redistribution.h"
#include "CalCommon.h"
#include "dsplot.h"

// General utility functions
#include "Tracer.h"
#include "stringutil.h"

// Local classes
#include "getcalresponse.h"

// ---------------------------------------------------

// Main function

// ---------------------------------------------------

void getcalresponse()
{
  
  PROC;

  /* Generate a CAL response */
  calServer.setInstrument(Xmm::EMOS1);
  calServer.state()->ccd()->set(EMOScam::CCD_1);
  calServer.state()->node()->set(EMOScam::PRIMARY_NODE);

  CanonicalRedist * rmf = calServer.getAtom(rmf);
  const BinnedAxis &ebounds = rmf->eBounds();
	
  cout << " Number of energy bins " << ebounds.bins() << endl;
 

  vector<double> response;
  string label = "Response for ";
 
  if (booleanParameter("userow"))
    {
      int row = intParameter("row");
      assert ( row >= 0 );
      unsigned int urow = (unsigned int) row;

      cout << " energy range covered (eV): " 
	   << ebounds.bin(urow).lo() << "  " << ebounds.bin(urow).hi()
	   << endl;
 
      assert ( urow < ebounds.bins() ); 

      response = rmf->spectrum(ebounds.bin(urow).mid());

      label += "row " + toString(row);
    }
  else
    {
      double energy = realParameter("energy");

//      cout << " row " << urow 
//	   << " found for requested energy " << energy << endl;
      cout << " energy range covered (eV): " 
	   << ebounds.bin(energy).lo() << "  " << ebounds.bin(energy).hi()
	   << endl;
 
      response = rmf->spectrum(energy);

      label += "energy " + toString(energy);
    }     
  
  /* Open output dataset and column */
  string outsetStr = stringParameter("outset");

  DataSet * set   = dataSetServer->open(outsetStr, 
					DataSet::Create);

  Table * table   = set->addTable("CAL_RESPONSE", response.size());

  Column * column = table->addColumn("RESPONSE",
				     Column::Real32, 
				     label);

  ColumnData * coldata = column->data();

  real32 * r32 = coldata->real32Data();

  /* Copy response data to column */
  unsigned int index = 0;
  for (vector<double>::const_iterator i = response.begin();
       i != response.end();
       ++i)
    {
      //      cout << "response ["<< index << "] " << *i << endl;
      r32[index] = *i;
      ++index;
    }
    
  dataSetServer->close(set);
  
  // Run dsplot if desired. Better to move this functionality to wrapper class.
  if (booleanParameter("display"))
    {
      cout << " Displaying result " << endl;
      openParameters("dsplot");
      setParameter("set", outsetStr);
      dsplot();
    }

  ENDPROC;

} 
