// This  skeleton file was automatically created by 
//            pkgmaker 0.1
//            Thursday, December 11 1997
// 
// Module: readrmf
// Author: Hassan Siddiqui
//
// Description: Expands a given row of the input rmf and writes 
// it out to a column in the output table. 
// 
// Comments: 
//  
// Modifications: See ChangeLog
// 
// $Id: readrmf.cc,v 1.7 2002/02/12 08:32:20 gvacanti Exp $

// General classes
#include <iostream>

// SAS-specific classes
#include "param.h"
#include "Dal.h"
#include "FileLocator.h"
#include "dsplot.h"

// General utility functions
#include "Tracer.h"

// Utility classes
#include "RmfRowToColumn.h"

// Local
#include "readrmf.h"

// ---------------------------------------------------

// Main function

// ---------------------------------------------------

/* NOTE: row indices correspond to FTOOLS approach, in that the first
 * row has an index of 1.
 */

void readrmf()
{
  PROC;

  // Get RMF file

  DataSet* rmfSet = 
    dataSetServer->open(stringParameter("table"), DataSet::Read);

  Table* rmfTable = rmfSet -> table(stringParameter("table"));


  // Open the output file
  string outsetStr = stringParameter("outset");
 
  DataSet* outSet = 
    dataSetServer->open(outsetStr, DataSet::Create);

  vector<LabelledInt> rows;
	
  if (stringParameter("gridtype")=="rows")
      {
	// Generate list of desired rows

	vector<string> rowNames = breakup(stringParameter("rows"));

	// Convert into a list of numbers
	
	for (vector<string>::const_iterator i = rowNames.begin();
	     i != rowNames.end();
	     ++i)
	  {
	    /* Add row to list, and add a label */
	    unsigned int userrow = atoi((*i).c_str());

	    if (userrow == 0)
	      {
		errstr << " Row found with index of 0. Rows start from 1. " 
		       << Sas::error("RowIndexZero");
		ALTRETURN;
	      }

	    /* Convert row number to C++ indexing */
	    unsigned int row = userrow-1;

	    if (row >= rmfTable->rows())
	      {
		errstr << " row " << userrow 
		       << " cannot be found in table " 
		       << Sas::error("RowNotFound");
		ALTRETURN;
	      }

	    rows.push_back(LabelledInt(row , "ROW_" + toString(userrow), "no label"));
	  }
      }
  else
    {
      // Generate list of desired energies

      vector<string> energyNames = breakup(stringParameter("energies"));

      unsigned int row = 0;

      VectorColumnContainer elowContainer(rmfTable->column("ENERG_LO"));

      for (vector<string>::const_iterator i = energyNames.begin();
	   i != energyNames.end();
	   ++i)
	{
	  // Convert into a number
	  double energy = atof((*i).c_str());
	  
	  // Go through table to find first row corresponding to that energy
	  while (true)
	    {
	      if (row >= rmfTable->rows())
		{
		  errstr << " Energy " << energy 
			 << " cannot be found in table " 
			 << Sas::error("EnergyNotFound");
		  ALTRETURN;
		}

	      if (elowContainer.value(row,0) > energy) break;
	      ++row;
	    }

	  rows.push_back(LabelledInt(row,
				     "ROW_" + toString(row + 1),
				     "Energy retrieved " + toString(elowContainer.value(row,0)) + 
				     " requested " + toString(energy)));
      	}
    }

  // Transpose rmf row(s) to output column.

  RmfRowToColumn::convert(rmfTable, 
			  outSet,
			  "RMF_DUMP",
			  rows);
  

  // Close datasets

  dataSetServer->close(rmfSet);
  dataSetServer->close(outSet);


  // Run dsplot if desired. Better to move this functionality to wrapper class.
  if (booleanParameter("display"))
    {
      cout << " Displaying result " << endl;
      openParameters("dsplot");
      setParameter("table", outsetStr);
      dsplot();
    }

  ENDPROC;
} 
